@@ -76,7 +76,7 @@ def compare(
             np.int32,
             np.int64,
             str,
-            np.bool,
+            bool,
             np.float16,
             np.double,
             np.uint32,
